/*****************************
* Source : pipes.prg           
* System : WAA/Xterm           
* Author : Phil Ide           
* Created: 29/11/2002             
*                             
* Purpose:                    
* ----------------------------
* History:                    
* ----------------------------
*    29/11/2002 13:04 PPI - Created                
*****************************/
                              
#include "dll.ch"
#include "Common.ch"
#include "pipes.ch"

//DLLFUNCTION CreatePipe(@nIn,@nOut,cSec,nLength) USING OSAPI FROM Kernel32.dll

CLASS UnNamedPipe
   PROTECTED:
      VAR nIn
      VAR nOut
      VAR nError

      CLASS VAR nKernel32

   EXPORTED:
      METHOD init
      METHOD write
      METHOD recv
      METHOD disconnect
      METHOD closeIn
      METHOD closeOut
      METHOD destroy
      METHOD error
      METHOD pipeIn
      METHOD pipeOut

      METHOD WriteFile
      METHOD ReadFile
      METHOD CloseHandle

ENDCLASS


METHOD UnNamedPipe:init(nIn,nOut)
   local pSecurity := CreateWin32APISecurityStructureForPipes()

   ::nIn  := 0
   ::nOut := 0

   if pcount() == 0
      ::nError := CreatePipe( @::nIn, @::nOut, pSecurity, 0 )
   else
      ::nIn := nIn
      ::nOut := nOut
   endif
   if ::nKernel32 == NIL
      ::nKernel32 := DllLoad('Kernel32.dll')
   endif
   return self

METHOD UnNamedPipe:write(xData)
   local nWritten := 0
   local lOk := FALSE
   local n

   if !::error(TRUE)
      n := ::WriteFile(xData,len(xData),@nWritten)
      lOk := ( n <> 0 )
   endif
   return lOk

METHOD UnNamedPipe:recv(nRead)
   local cBuff := ''
   local n
   local lOk := FALSE
   local nBytesRead := 0

   if !::error(TRUE)
      default nRead to 1024

      cBuff := Space(nRead)+Chr(0)

      n := ::ReadFile(@cBuff,nRead,@nBytesRead)
      lOk := (n <> 0)
      if !lOk
         cBuff := ''
      else
         cBuff := left(cBuff,nBytesRead)
      endif
   endif
   return cBuff


METHOD UnNamedPipe:pipeIn(lAsString)
   local xRet := ::nIn
   default lAsString to FALSE
   if lAsString
      xRet := LTrim( Str( xRet ) )
   endif
   return xRet

METHOD UnNamedPipe:pipeOut(lAsString)
   local xRet := ::nOut
   default lAsString to FALSE
   if lAsString
      xRet := LTrim( Str( xRet ) )
   endif
   return xRet

// oUPipe:error()     -> nErr // error value set when creating pipe
// oUPipe:error(TRUE) -> lErr // TRUE if error when creating pipe
METHOD UnNamedPipe:error(lLogical)
   local xRet
   default lLogical to FALSE
   if lLogical
      xRet := ::nError == 0
   else
      xRet := ::nError
   endif
   return (xRet)

METHOD UnNamedPipe:closeIn()
   if !::error(TRUE)
      ::CloseHandle(::nIn)
   endif
   return (self)
METHOD UnNamedPipe:closeOut()
   if !::error(TRUE)
      ::CloseHandle(::nOut)
   endif
   return (self)
METHOD UnNamedPipe:disconnect()
   if !::error(TRUE)
      ::closeIn()
      ::closeOut()
   endif
   return (self)

METHOD UnNamedPipe:destroy()
   ::close()
   return (self)

METHOD UnNamedPipe:WriteFile( cBuff, nLen, nWritten )
   STATIC cTemplate

   if cTemplate == NIL
      cTemplate := DllPrepareCall(::nKernel32,DLL_OSAPI,'WriteFile')
   endif
   return DllExecuteCall(cTemplate,::nOut, @cBuff, nLen, @nWritten, NIL )

METHOD UnNamedPipe:ReadFile( cBuff, nLen, nRead )
   STATIC cTemplate

   if cTemplate == NIL
      cTemplate := DllPrepareCall(::nKernel32,DLL_OSAPI,'ReadFile')
   endif
   return DllExecuteCall(cTemplate,::nIn, @cBuff, nLen, @nRead, NIL )

METHOD UnNamedPipe:CloseHandle(nHnd)
   STATIC cTemplate

   if cTemplate == NIL
      cTemplate := DllPrepareCall(::nKernel32,DLL_OSAPI,'CloseHandle')
   endif
   return DllExecuteCall(cTemplate,nHnd)

