/*****************************
* Source : sockets.prg           
* System : <unkown>           
* Author : Phil Ide           
* Created: 30/12/2002             
*                             
* Purpose:                    
* ----------------------------
* History:                    
* ----------------------------
*    30/12/2002 11:18 PPI - Created                
*****************************/
                              
#include "Common.ch"
#include "Asinetc.ch"
#include "DLL.CH"

CLASS SocketsBase
   PROTECTED:
      VAR nSocket
      VAR cSocket
      VAR nIPAddress
      VAR cPeerName
      VAR lPeerFixed

      METHOD receive

   EXPORTED:
      VAR cargo

      METHOD init
      METHOD write
      METHOD recv
      METHOD disconnect
      METHOD peek
      METHOD pipeName
      METHOD getPeerName

ENDCLASS


METHOD SocketsBase:init(nSocket)
   ::nSocket := nSocket
   ::cSocket := LTrim( Str( nSocket ) )
   ::lPeerFixed := FALSE
   return (self)

METHOD SocketsBase:getPeerName()
   local nError := 0
   local aPeer
   local aHostInfo
   local aRet[2]

   if (::nIpAddress == NIL)
      aPeer := SocketGetPeerName(::nSocket,@nError)

      if !(aPeer == NIL)
         ::nIpAddress := aPeer[IPADDR_ADDR]
      endif
   endif

   if (::cPeerName == NIL) .and. !(::nIpAddress == NIL)
      nError := 0
      aHostInfo := SocketGetHostByAddr(::nIpAddress,@nError)

      if !(aHostInfo == NIL)
         ::cPeerName := aHostInfo[HOSTINFO_CNAME]
      endif
   endif

   ::cPeerName  := iif( ::cPeerName == NIL,'<unknown>', ::cPeerName )
   if !::lPeerFixed
      ::nIpAddress := iif( ::nIpAddress == NIL, '0.0.0.0', SocketINetNtoA(::nIpAddress) )
      ::lPeerFixed := TRUE
   endif

   aRet[1] := ::nIpAddress
   aRet[2] := ::cPeerName
   return aRet

METHOD SocketsBase:disconnect()
   SocketClose(::nSocket)
   return (self)

METHOD SocketsBase:peek()
   return TRUE

METHOD SocketsBase:pipeName()
   return ::cSocket

METHOD SocketsBase:write(cData)
   local cBuff    := Var2Stream(cData)
   local cLength  := U2Bin(len(cBuff))
   local nLength  := Len(cLength)+Len(cBuff)                                                           // Actual bytes to write
   local nWrite   := 0                                                                    // Actual written
   local nTotal

   cBuff := cLength+cBuff
   nWrite         := SocketSend(::nSocket,cBuff)                                          // Try to write in one pass
   if nWrite < nLength                                                                    // Unable to do so.....
      nTotal      := nWrite                                                               // How meny bytes left ?
      while nTotal < nLength                                                              // Until somthing to write
         cData    := Right(cBuff,nLength - nTotal)                                        // New output buffer
         nWrite   := SocketSend(::nSocket,cData)                                          // Next write
         nTotal   += nWrite                                                               // Actual bytes written
      enddo
   endif
   Return nLength

METHOD SocketsBase:recv(nError)
   local nLen
   local lOk := FALSE
   local cData

   cData := ''
   nLen := 0

   if ::receive(@nLen, 4, MSG_NORMAL, @nError) == 4
      nLen := Bin2U(nLen)
      if ::receive(@cData, nLen, MSG_NORMAL, @nError) == nLen
         lOk   := TRUE
      endif
   endif
   if !lOk
      cData := '' // ensure it is rubbish
      if nError == 0
         nError++
      endif
   else
      cData := Stream2Var(cData)
   endif
   return cData

METHOD SocketsBase:receive(cBuffer, nBytes, nType, nError)
   local nRead                                                                               // Bytes actual read
   local nTotal, cRest
   local nIdle                                                                               // 04/12/2002 PI - Blocking-mode off
   local nWait                                                                               // 08/12/2002 J. Duijf, timout
   local nTimeOut    := 2                                                                    // Timout = 2 seconds

   cBuffer := Space( nBytes )                                                                // Create READ buffer
   nRead  := SocketRecv( ::nSocket, @cBuffer, nBytes, nType, @nError )                         // Try to read in one pass
   if nRead < nBytes                                                                         // Unable to read all...
      nTotal := nRead                                                                        // How many bytes are missing ??
      nWait       := Seconds() + nTimeOut                                                    // GC timeout
      if nWait > 86399.99                                                                    // later then 23:59:59:99
         nWait    := nWait - 86399.99                                                        // Correct for the next day
      endif
      do while nTotal < nBytes                                                               // Read until all bytes arrived
         cBuffer  := Padr(cBuffer,nTotal)                                                    // Adjust buffer
         cRest    := Space(nBytes - nTotal)                                                  // New buffer to read rest
         nRead    := SocketRecv( ::nSocket, @cRest, nBytes - nTotal, nType, @nError )          // Try to read rest of data
         if nRead = 0                                                                        // No data read
            if nWait < Seconds()                                                             // Timeout, activate GC
               Sleep(1)                                                                      
               nWait := Seconds() - nTimeOut                                                 // Prevent 99% utilisation after midnight
            endif
         else
            nWait    := Seconds() + 2                                                        // 2 seconds timeout
            if nWait > 86399.99
               nWait    := nWait - 86399.99
            endif
            nTotal   += nRead                                                                // Total read bytes
            cBuffer  += cRest                                                                // Add to current buffer
         endif
      enddo
   endif
   cRest := NIL                                                                              // Release memory
   Return nBytes                                                                             // Actual bytes read

